/*
    Irchat - Infrared chat on TI89 calculators
    Copyright (C) 2003  Sbastien Bourdeauducq

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "config.h"

#include <link.h>
#include <mem.h>
#include <peekpoke.h>
#include <system.h>

#include "comm_lowlevel.h"

void uart_init() {
  // initialize the link interface for low-level access
  pokeIO(LINK_REGISTER, 0);
  pokeIO_bset(LINK_REGISTER, 6);
  pokeIO_bclr(LINK_PORT, 0);
  pokeIO_bclr(LINK_PORT, 1);

  pokeIO(0x600017, 0xB2);
  ir_off();
}

void uart_free() {
  OSLinkReset();
}

unsigned char hwtest() {
  OSFreeTimer(USER_TIMER);
  OSRegisterTimer(USER_TIMER, 2);
  ir_off();
  while (!OSTimerExpired(USER_TIMER));
  if (ir_read()) return FALSE;
  ir_on();
  while (!OSTimerExpired(USER_TIMER));
  if (!ir_read()) {
    ir_off();
    while (!OSTimerExpired(USER_TIMER));
    return FALSE;
  }
  ir_off();
  while (!OSTimerExpired(USER_TIMER));
  return TRUE;
}

#define BIT_LENGTH    4
#define BIT_LENGTH_54 5

unsigned char uart_send(void *buffer, unsigned short length) {
  unsigned char *current;
  unsigned short i;
  unsigned char j;

  current = buffer;
  pokeIO(0x600017, 0xFF);
  ir_on();
  OSFreeTimer(USER_TIMER);
  OSRegisterTimer(USER_TIMER, 90);
  while(!OSTimerExpired(USER_TIMER));
  if (!ir_read()) {
    pokeIO(0x600017, 0xB2);
    OSSetSR(0x0000);
    ir_off();
    return FALSE;
  }
  OSFreeTimer(USER_TIMER);

  OSRegisterTimer(USER_TIMER, BIT_LENGTH);

  ir_off();
  while (!OSTimerExpired(USER_TIMER));

  for(i=0; i<length; i++) {
    j=0b00000001;
    while(j<0b10000000) {
      if((*current & j) == j) ir_on(); else ir_off();
      while (!OSTimerExpired(USER_TIMER));
      j = j << 1;
    }
    if((*current & j) == j) ir_on(); else ir_off();
    while (!OSTimerExpired(USER_TIMER));
    current++;
  }

  pokeIO(0x600017, 0xB2);
  ir_off();
  return TRUE;
}

unsigned char uart_recv(void *buffer, unsigned short length) {
  char *current;
  unsigned short i, j;

  memset(buffer, 0, length);
  current = buffer;
  pokeIO(0x600017, 0xFF);
  while(ir_read());
  OSFreeTimer(USER_TIMER);
  OSRegisterTimer(USER_TIMER, BIT_LENGTH_54);
  while(!OSTimerExpired(USER_TIMER));
  OSFreeTimer(USER_TIMER);
  OSRegisterTimer(USER_TIMER, BIT_LENGTH);

  for(i=0; i<length; i++) {
    for(j=1; j<=128; j *= 2) {
      if(ir_read()) *current |= j;
      while(!OSTimerExpired(USER_TIMER));
    }
    current++;
  }

  pokeIO(0x600017, 0xB2);
  return TRUE;
}

