/*
    Irchat - Infrared chat on TI89 calculators
    Copyright (C) 2003  Sbastien Bourdeauducq

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "config.h"

#include <dialogs.h>
#include <graph.h>
#include <kbd.h>
#include <system.h>
#include <statline.h>
#include <wingraph.h>

#include "chatroom.h"
#include "comm_lowlevel.h"
#include "icon.h"
#include "icon_chat.h"
#include "icon_config.h"
#include "icon_exit.h"
#include "locale.h"
#include "settings.h"

/* Only for TI-89 !!! */
#define KEY_UP 337
#define KEY_DOWN 340

void *kbq;
unsigned char current_sel;

unsigned short getkey()
{
  unsigned short key;
  while(OSdequeue(&key, kbq)) idle();
  return key;
}

#define XOR1 WinFill(DeskTop, &(WIN_RECT){21, 39, 139, 49}, A_XOR)
#define XOR2 WinFill(DeskTop, &(WIN_RECT){21, 55, 139, 65}, A_XOR)
#define XOR3 WinFill(DeskTop, &(WIN_RECT){21, 71, 139, 81}, A_XOR)

void draw_screen()
{
  WinClr(DeskTop);
  ST_busy(ST_IDLE);

  WinBitmapPut(DeskTop, 0, 0, &icon, A_REPLACE);
  WinFont(DeskTop, F_8x10);
  WinStrXY(DeskTop, 33, 0, "Irchat "VERSION);
  WinFont(DeskTop, F_6x8);
  WinStrXY(DeskTop, 33, 11, "(C) 2003 Seb");
  WinFont(DeskTop, F_4x6);
  WinStrXY(DeskTop, 33, 22, "http://irchat.free.fr/");
  WinLineNC(DeskTop, &(WIN_RECT){0, 32, 160, 32});

  WinRect(DeskTop, &(WIN_RECT){20, 38, 140, 50}, B_NORMAL | A_NORMAL);
  WinAttr(DeskTop, A_REVERSE);
  WinPixSet(DeskTop, 20, 38);
  WinPixSet(DeskTop, 20, 50);
  WinPixSet(DeskTop, 140, 38);
  WinPixSet(DeskTop, 140, 50);
  WinAttr(DeskTop, A_NORMAL);
  WinBitmapPut(DeskTop, 23, 39, &icon_chat, A_REPLACE);
  WinStrXY(DeskTop, 40, 42, START_CHATTING);

  WinRect(DeskTop, &(WIN_RECT){20, 54, 140, 66}, B_NORMAL | A_NORMAL);
  WinAttr(DeskTop, A_REVERSE);
  WinPixSet(DeskTop, 20, 54);
  WinPixSet(DeskTop, 20, 66);
  WinPixSet(DeskTop, 140, 54);
  WinPixSet(DeskTop, 140, 66);
  WinAttr(DeskTop, A_NORMAL);
  WinBitmapPut(DeskTop, 23, 55, &icon_config, A_REPLACE);
  WinStrXY(DeskTop, 40, 58, SETTINGS_MENU);

  WinRect(DeskTop, &(WIN_RECT){20, 70, 140, 82}, B_NORMAL | A_NORMAL);
  WinAttr(DeskTop, A_REVERSE);
  WinPixSet(DeskTop, 20, 70);
  WinPixSet(DeskTop, 20, 82);
  WinPixSet(DeskTop, 140, 70);
  WinPixSet(DeskTop, 140, 82);
  WinAttr(DeskTop, A_NORMAL);
  WinBitmapPut(DeskTop, 23, 71, &icon_exit, A_REPLACE);
  WinStrXY(DeskTop, 40, 74, EXIT);

  current_sel = 1;
  XOR1;
  ST_showHelp(START_CHATTING_HELP);
}

void _main(void)
{
  LCD_BUFFER lcdbuffer;
  unsigned char exit;

  SetAlphaStatus(TRUE);

  LCD_save(lcdbuffer);

  kbq = kbd_queue();

  load_settings();

  uart_init();

  /* Setup the screen */
  DeskTop->Client.xy.y0 = 0;
  DeskTop->Window.xy.y0 = 0;
  DeskTop->Clip.xy.y0 = 0;

  draw_screen();

  exit = FALSE;

  while(!exit) {
    switch(getkey()) {
      case KEY_UP:
        if(current_sel != 1) {
          if(current_sel == 2) XOR2;
          if(current_sel == 3) XOR3;
          current_sel--;
          if(current_sel == 1) { XOR1; ST_showHelp(START_CHATTING_HELP); };
          if(current_sel == 2) { XOR2; ST_showHelp(SETTINGS_HELP); }
        }
        break;
      case KEY_DOWN:
        if(current_sel != 3) {
          if(current_sel == 1) XOR1;
          if(current_sel == 2) XOR2;
          current_sel++;
          if(current_sel == 2) { XOR2; ST_showHelp(SETTINGS_HELP); }
          if(current_sel == 3) { XOR3; ST_showHelp(EXIT_HELP); }
        }
        break;
      case KEY_ENTER:
        switch(current_sel) {
          case 1:
            if(hwtest()) {
              chatroom_start();
              draw_screen();
            } else {
              DlgMessage(HW_PROBLEM, HW_NODETECT, BT_OK, BT_NONE);
              ST_busy(ST_IDLE);
              ST_showHelp(START_CHATTING_HELP);
            }
            break;
          case 2:
            show_config_dialog();
            ST_showHelp(SETTINGS_HELP);
            ST_busy(ST_IDLE);
            break;
          case 3:
            exit = TRUE;
            break;
        }
        break;
      case KEY_ESC:
        exit = TRUE;
        break;
    }
  }

  /* Cleanup the screen */
  DeskTop->Client.xy.y0 = 18;
  DeskTop->Window.xy.y0 = 18;
  DeskTop->Clip.xy.y0 = 18;

  uart_free();

  LCD_restore(lcdbuffer);
  SetAlphaStatus(FALSE);
  ST_eraseHelp();
}
